@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 1997-1999 The Apache Group.  All rights reserved.
+ * Copyright (c) 1997-2002 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -49,13 +49,10 @@
  * project, please see <http://www.apache.org/>.
  *
  *
- *  CVS $Id: mod_auth_radius.c,v 1.4 2000/08/25 14:15:36 aland Exp $
+ *  CVS $Id: mod_auth_radius.c,v 1.14 2002/09/10 14:12:36 aland Exp $
  */
 
 /*
-  Version 1.5.0, Nov. 1999, by Alan DeKok <alan@freeradius.org>
-  for CRYPTOCard Inc.          http://www.cryptocard.com/
-
   Everyone wants strong authentication over the web.  For us, this means
   RADIUS.
   
@@ -72,6 +69,25 @@
   Also, do NOT have your RADIUS server visible to the external world.
   Doing so makes all kinds of attacks possible.
 
+  **************************************************
+  
+  Add to Configuration file BEFORE mod_auth.o:
+  Module radius_auth_module    mod_auth_radius.o
+  
+  Add to server configuration file
+  AddRadiusAuth <server>[:port] <secret> [<seconds>[:<retries>]]
+  AddRadiusCookieValid <minutes>
+  AddModule modules/extra/mod_auth_radius.o              (for 1.3.x)
+  
+  Add to directory configuration
+  AddRadiusAuth <server>[:port] <secret> [<seconds>]
+  AuthRadiusBindAddress <local address/interface>
+  AuthRadiusAuthoritative on
+  AuthRadiusActive on
+  AuthRadiusCookieValid <minutes>
+
+  **************************************************
+
   Adding mod_auth_radius to the Configuration file before mod_auth
   allows you to have mod_auth_radius authoritative by default, but NOT
   have it interfere with the rest of your configuration.  The authentication
@@ -217,6 +233,24 @@
   Version History
   ===============
 
+  1.5.4  Support for retries from John Lines <john.lines@integris.co.uk>
+
+  1.5.3 Bug fix from Bryan Stansell <bryan@stansell.org>, to set
+        the right data element for the AddRadiusCookieValid configuration
+	item.
+
+  1.5.2 Updates for NAS-Identifier and NAS-IP-Address, based on ideas
+        from Adrian Hosey <ahosey@systhug.com>.  The NAS-Identifier is
+	the virtual server host name, and the NAS-IP-Address is the
+	IP address of the base server.
+
+	Also integrated code from http://www.wede.de/sw/mod_auth_radius/
+        which had forked form this one after v1.3.3.
+
+  1.5.1 Quick release, for bug found by f.garosi@usl7.toscana.it.
+
+  1.5.0 Don't stat() proxy requests.
+
   1.3.3 Another minor bug fix and configuration hints for Apache 1.3.x
         Thanks to Hiroshi MIZOGUCHI <mizoguti@screen.co.jp>.
 
@@ -306,11 +340,12 @@ typedef struct radius_packet_t {
 /* RADIUS attribute definitions. Also from RFC 2138 */
 #define	RADIUS_USER_NAME	      1
 #define	RADIUS_PASSWORD		      2
-#define	RADIUS_CLIENT_ID	      4
+#define	RADIUS_NAS_IP_ADDRESS	      4
 #define RADIUS_SERVICE_TYPE           6
 #define RADIUS_REPLY_MESSAGE          18
 #define RADIUS_STATE		      24
 #define RADIUS_SESSION_TIMEOUT        27
+#define	RADIUS_NAS_IDENTIFIER	      32
 
 /* service types : authenticate only for now */
 #define RADIUS_AUTHENTICATE_ONLY      8
@@ -335,7 +370,10 @@ typedef struct radius_server_config_struct {
   int secret_len;		/* length of the secret (to save time later) */
   int timeout;			/* cookie valid time */
   int wait;			/* wait for RADIUS server responses */
+  int retries;			/*  number of retries on timeout */
   unsigned short port;		/* RADIUS port number */
+  unsigned long bind_address;	/* bind socket to this local address */
+  struct radius_server_config_struct *next; /* fallback server(s) */
 } radius_server_config_rec;
 
 /* per-server configuration create */
@@ -350,7 +388,10 @@ create_radius_server_config(pool *p, server_rec *s)
   scr->secret = NULL;		/* no secret yet */
   scr->secret_len = 0;
   scr->wait = 5;		/* wait 5 sec before giving up on the packet */
+  scr->retries = 0;		/* no additional retries */
   scr->timeout = 60;		/* valid for one hour by default */
+  scr->bind_address = INADDR_ANY;
+  scr->next = NULL;
   return scr;
 }
 
@@ -391,17 +432,39 @@ get_random_vector(unsigned char vector[RADIUS_RANDOM_VECTOR_LEN])
   MD5Final(vector, &my_md5);	      /* set the final vector */
 }
 
+/* Per-dir configuration structure */
+typedef struct radius_dir_config_struct {
+  radius_server_config_rec* server;
+  int active;                   /* Are we doing RADIUS in this dir? */
+  int authoritative;		/* is RADIUS authentication authoritative? */
+  int timeout;			/* cookie time valid */
+} radius_dir_config_rec;
+
+/* Per-dir configuration create */
+static void *
+create_radius_dir_config (pool *p, char *d)
+{
+  radius_dir_config_rec *rec =
+    (radius_dir_config_rec *) pcalloc (p, sizeof(radius_dir_config_rec));
+
+  rec->server = NULL;		/* no valid server by default */
+  rec->active = 1;              /* active by default */  
+  rec->authoritative = 1;	/* authoritative by default */
+  rec->timeout = 0;		/* let the server config decide timeouts */
+  return rec;
+}
+
 /* per-server set configuration */
 static const char *
-add_auth_radius(cmd_parms *cmd, void *mconfig, char *server, char *secret, char *wait)
+add_auth_radius(cmd_parms *cmd, void *mconfig,
+		char *server, char *secret, char *wait)
 {
   radius_server_config_rec *scr;
   unsigned int port;
   char *p;
 
-  scr = get_module_config(cmd->server->module_config,
-			  &radius_auth_module);
-  
+  scr = get_module_config(cmd->server->module_config, &radius_auth_module);
+
   /* allocate and look up the RADIUS server's IP address */
   scr->radius_ip = palloc(cmd->pool, sizeof(struct in_addr));
 
@@ -422,28 +485,46 @@ add_auth_radius(cmd_parms *cmd, void *mconfig, char *server, char *secret, char
   scr->secret = pstrdup(cmd->pool, secret);
   scr->secret_len = strlen(scr->secret);
   if (wait != NULL) {
+    if ((p = strchr(wait,':')) != NULL) {
+      *(p++) = 0;   /* null terminate the wait part of the string */
+      scr->retries = atoi(p);
+    } 
     scr->wait = atoi(wait);
   } /* else it's already initialized */
+  scr->bind_address = INADDR_ANY;
 
   return NULL;			/* everything's OK */
 }
 
-/* Per-dir configuration structure */
-typedef struct radius_dir_config_struct {
-  int authoritative;		/* is RADIUS authentication authoritative? */
-  int timeout;			/* cookie time valid */
-} radius_dir_config_rec;
-
-/* Per-dir configuration create */
-static void *
-create_radius_dir_config (pool *p, char *d)
+/*
+ *  Set the local address to which this client is bound.
+ */
+static const char *
+set_bind_address (cmd_parms *cmd, void *mconfig, char *arg)
 {
-  radius_dir_config_rec *rec =
-    (radius_dir_config_rec *) pcalloc (p, sizeof(radius_dir_config_rec));
-  
-  rec->authoritative = 1;	/* authoritative by default */
-  rec->timeout = 0;		/* let the server config decide timeouts */
-  return rec;
+  radius_server_config_rec *scr;
+  struct in_addr *a;
+
+  scr = get_module_config(cmd->server->module_config,
+                                &radius_auth_module);
+  if ((a = get_ip_addr(cmd->pool, arg)) == NULL)
+      return "AuthRadiusBindAddress: invalid IP address";
+  scr->bind_address = a->s_addr;
+  return NULL;
+}
+
+/*
+ *  Set the cookie valid time.
+ */
+static const char *
+set_cookie_valid(cmd_parms *cmd, void *mconfig, char *arg)
+{
+  radius_server_config_rec *scr;
+
+  scr = get_module_config(cmd->server->module_config,
+			  &radius_auth_module);
+  scr->timeout = atoi(arg);
+  return NULL;			/* everything's OK */
 }
 
 static const char *
@@ -457,22 +538,28 @@ set_int_slot (cmd_parms *cmd, char *struct_ptr, char *arg)
 /* Table of which command does what */
 static command_rec auth_cmds[] = {
   { "AddRadiusAuth", add_auth_radius, NULL, RSRC_CONF, TAKE23,
-    "radius server name:port,  shared secret, and optional timeout" },
+    "per-server configuration for RADIUS server name:port,  shared secret, and optional timeout:retries" },
   
-  { "AddRadiusCookieValid", set_int_slot, 
-    (void*)XtOffsetOf(radius_server_config_rec, timeout), 
-    RSRC_CONF, TAKE1,
-    "time in minutes that the returned cookie is valid for.  After this time, authentication will be requested again.  Use '0' for forever." }, 
+  { "AuthRadiusBindAddress", set_bind_address, NULL, RSRC_CONF, TAKE1,
+    "per-server binding local socket to this local IP address.  RADIUS requests will be sent *from* this IP address." },
+
+  { "AddRadiusCookieValid", set_cookie_valid, NULL, RSRC_CONF, TAKE1,
+    "per-server time in minutes for which the returned cookie is valid.  After this time, authentication will be requested again.  Use '0' for forever." }, 
 
   { "AuthRadiusAuthoritative", set_flag_slot,
     (void*)XtOffsetOf(radius_dir_config_rec, authoritative), 
     OR_AUTHCFG, FLAG, 
-    "Set to 'no' to allow access control to be passed along to lower modules if the UserID is not known to this module" },
+    "per-directory access on failed authentication.  If set to 'no', then access control is passed along to lower modules on failed authentication." },
 
   { "AuthRadiusCookieValid", set_int_slot, 
     (void*)XtOffsetOf(radius_dir_config_rec, timeout), 
     OR_AUTHCFG, TAKE1,
-    "time in minutes that the returned cookie is valid for.  After this time, authentication will be requested again.  Use '0' for forever." }, 
+    "per-directory time in minutes for which the returned cookie is valid.  After this time, authentication will be requested again.  Use '0' for forever." }, 
+  
+  { "AuthRadiusActive", set_flag_slot,
+    (void*)XtOffsetOf(radius_dir_config_rec, active),
+    OR_AUTHCFG, FLAG,
+    "per-directory toggle the use of RADIUS authentication." },
 
   { NULL }
 };
@@ -489,7 +576,8 @@ xor(unsigned char *p, unsigned char *q, int length)
 }
 
 static int
-verify_packet(request_rec *r, radius_packet_t *packet, unsigned char vector[RADIUS_RANDOM_VECTOR_LEN])
+verify_packet(request_rec *r, radius_packet_t *packet,
+	      unsigned char vector[RADIUS_RANDOM_VECTOR_LEN])
 {
   server_rec *s = r->server; 
   radius_server_config_rec *scr = (radius_server_config_rec *)
@@ -560,6 +648,22 @@ make_cookie(request_rec *r, time_t expires, const char *passwd, const char *stri
    * Oh well, at least the connection is traceable to a particular machine.
    */
 
+  /*
+   *  Piotr Klaban <makler@oryl.man.torun.pl> says:
+   *
+   *  > The "squid" proxy set HTTP_X_FORWARDED_FOR variable - the
+   *  > original IP of the client.  We can use HTTP_X_FORWARDED_FOR
+   *  > variable besides REMOTE_ADDR.
+   *
+   *  > If cookie is stolen, then atacker could use the same proxy as
+   *  > the client, to validate the cookie. If we would use
+   *  > HTTP_X_FORWARDED_FOR, then useing the proxy would not be
+   *  > sufficient.
+   *
+   *  We don't do this, mainly because I haven't gotten around to
+   *  writing the code...
+   */
+
   /*
    * Make a cookie based on secret + public information.
    *
@@ -640,11 +744,21 @@ spot_cookie(request_rec *r)
   const char *cookie;
   char *value;
 
-  if ((cookie = table_get (r->headers_in, "Cookie"))) {
+  if ((cookie = table_get(r->headers_in, "Cookie"))) {
     if ((value=strstr(cookie, cookie_name))) {
       char *cookiebuf, *cookieend;
       
-      value += strlen(cookie_name) + 1; /* skip '=' */
+      value += strlen(cookie_name); /* skip the name */
+
+      /*
+       *  Ensure there's an '=' after the name.
+       */
+      if (*value != '=') {
+	return NULL;
+      } else {
+	value++;
+      }
+      
       cookiebuf = pstrdup( r->pool, value );
       cookieend = strchr(cookiebuf,';');
       if (cookieend) *cookieend = '\0';	/* Ignore anything after a ; */
@@ -667,15 +781,16 @@ radius_authenticate(request_rec *r, radius_server_config_rec *scr,
   struct sockaddr saremote;
   int salen, total_length;
   fd_set set;
+  int retries = scr->retries;
   struct timeval tv;
   int rcode;
+  struct in_addr *ip_addr;
   
   unsigned char misc[RADIUS_RANDOM_VECTOR_LEN];
   int password_len, i;
   unsigned char password[128];
   AP_MD5_CTX md5_secret, my_md5;
   UINT4 service;
-  struct in_addr *ip_addr;
 
   unsigned char send_buffer[RADIUS_PACKET_SEND_SIZE];
   radius_packet_t *packet = (radius_packet_t *) send_buffer;
@@ -734,16 +849,27 @@ radius_authenticate(request_rec *r, radius_server_config_rec *scr,
 		sizeof(service));
   
   /* ************************************************************ */
-  /* Tell the RADIUS server where we're coming from */
+  /* Tell the RADIUS server which virtual server we're coming from */
+  add_attribute(packet, RADIUS_NAS_IDENTIFIER, r->server->server_hostname,
+		strlen(r->server->server_hostname));
+
+  /* ************************************************************ */
+  /* Tell the RADIUS server which IP address we're coming from */
   if (scr->radius_ip->s_addr == htonl(0x7f000001)) {
     ip_addr = scr->radius_ip; /* go to localhost through localhost */
-  } else if ((ip_addr = get_ip_addr(r->pool, r->server->server_hostname)) == NULL) {
-    ap_snprintf(errstr, MAX_STRING_LEN, "cannot look up server hostname %s", r->server->server_hostname);
-    return FALSE;
+  } else {
+    ip_addr = get_ip_addr(r->pool, r->connection->base_server->server_hostname);
+    if (ip_addr == NULL) {
+      ap_snprintf(errstr, MAX_STRING_LEN, "cannot look up server hostname %s",
+		  r->connection->base_server->server_hostname);
+      return FALSE;
+    }
   }
-  add_attribute(packet, RADIUS_CLIENT_ID, (unsigned char *)&ip_addr->s_addr,
+
+  add_attribute(packet, RADIUS_NAS_IP_ADDRESS, (unsigned char *)&ip_addr->s_addr,
 		sizeof(ip_addr->s_addr));
   
+  
   /* ************************************************************ */
   /* add state, if requested */
   if (state != NULL) {
@@ -761,29 +887,51 @@ radius_authenticate(request_rec *r, radius_server_config_rec *scr,
   sin->sin_addr.s_addr = scr->radius_ip->s_addr;
   sin->sin_port = htons(scr->port);
 
-  if (sendto(sockfd, (char *) packet, total_length, 0,
-	     &saremote, sizeof(struct sockaddr_in)) < 0) {
-    ap_snprintf(errstr, MAX_STRING_LEN, "error sending RADIUS packet for user %s: %s", user, strerror(errno));
-    return FALSE;
-  }
-  
-  /* ************************************************************ */
-  /* Wait for the response, and verify it. */
-  salen = sizeof (saremote);
-  tv.tv_sec = scr->wait;	/* wait for the specified time */
-  tv.tv_usec = 0;
-  FD_ZERO(&set);		/* clear out the set */
-  FD_SET(sockfd, &set);	/* wait only for the RADIUS UDP socket */
-  
-  rcode = select(sockfd + 1, &set, NULL, NULL, &tv);
-  if (rcode == 0) {		/* time out */
-    ap_snprintf(errstr, MAX_STRING_LEN, "RADIUS server failed to respond within %d seconds", scr->wait);
-    return FALSE;
-  } else if (rcode < 0) {	/* error */
+  while (retries >= 0) {
+    if (sendto(sockfd, (char *) packet, total_length, 0,
+	       &saremote, sizeof(struct sockaddr_in)) < 0) {
+      ap_snprintf(errstr, MAX_STRING_LEN, "error sending RADIUS packet for user %s: %s", user, strerror(errno));
+      return FALSE;
+    }
+
+  wait_again:
+    /* ************************************************************ */
+    /* Wait for the response, and verify it. */
+    salen = sizeof (saremote);
+    tv.tv_sec = scr->wait;	/* wait for the specified time */
+    tv.tv_usec = 0;
+    FD_ZERO(&set);		/* clear out the set */
+    FD_SET(sockfd, &set);	/* wait only for the RADIUS UDP socket */
+    
+    rcode = select(sockfd + 1, &set, NULL, NULL, &tv);
+    if ((rcode < 0) && (errno == EINTR)) {
+      goto wait_again;		/* signal, ignore it */
+    }
+
+    if (rcode == 0) {		/* done the select, with no data ready */
+      retries--;
+    } else {
+      break;			/* exit from the 'while retries' loop */
+    }
+  } /* loop over the retries */
+
+  /*
+   *  Error.  Die.
+   */
+  if (rcode < 0) {
     ap_snprintf(errstr, MAX_STRING_LEN, "error waiting for RADIUS response: %s", strerror(errno));
     return FALSE;
   }
   
+  /*
+   *  Time out.
+   */
+  if (rcode == 0) {
+    ap_snprintf(errstr, MAX_STRING_LEN, "RADIUS server %s failed to respond within %d seconds after each of %d retries",
+		inet_ntoa(*scr->radius_ip), scr->wait, scr->retries);
+    return FALSE;
+  }
+
   if ((total_length = recvfrom(sockfd, (char *) recv_buffer,
 			       RADIUS_PACKET_RECV_SIZE,
 			       0, &saremote, &salen)) < 0) {
@@ -852,7 +1000,7 @@ check_pw(request_rec *r, radius_server_config_rec *scr, const char *user, const
   sin = (struct sockaddr_in *) &salocal;
   memset((char *) sin, '\0', sizeof(salocal));
   sin->sin_family = AF_INET;
-  sin->sin_addr.s_addr = INADDR_ANY;
+  sin->sin_addr.s_addr = scr->bind_address;
   
   local_port = 1025;
   do {
@@ -899,9 +1047,7 @@ check_pw(request_rec *r, radius_server_config_rec *scr, const char *user, const
       break;
       
     case RADIUS_ACCESS_CHALLENGE:
-      if (state != NULL) {    /* Complain if the state already exists */
-	ap_snprintf(errstr, MAX_STRING_LEN, "RADIUS returned Access-Challenge when sent State (authentication failure)");
-      } else {			/* it's good: set the state */
+      {
 	attribute_t *a_state, *a_reply;
 	time_t expires = time(NULL) + 120; /* state expires in two minutes */
 	char server_state[256];
@@ -928,17 +1074,21 @@ check_pw(request_rec *r, radius_server_config_rec *scr, const char *user, const
 	  }
 	  
 	  /* set the magic cookie */
-	  add_cookie(r, r->err_headers_out, make_cookie(r, expires, "", server_state), expires);
+	  add_cookie(r, r->err_headers_out,
+		     make_cookie(r, expires, "", server_state), expires);
 
 	  /* log the challenge, as it IS an error returned to the user */
-	  ap_snprintf(errstr, MAX_STRING_LEN, "RADIUS server requested challenge for user %s", user);
+	  ap_snprintf(errstr, MAX_STRING_LEN,
+		      "RADIUS server requested challenge for user %s", user);
 
 	}
       }
       break;
       
     default:			/* don't know what else to do */
-      ap_snprintf(errstr, MAX_STRING_LEN, "RADIUS server returned unknown response %02x", packet->code);
+      ap_snprintf(errstr, MAX_STRING_LEN,
+		  "RADIUS server returned unknown response %02x",
+		  packet->code);
       break;
     }
   
@@ -987,8 +1137,8 @@ authenticate_basic_user(request_rec *r)
   time_t expires;
   struct stat buf;
   
-  if (!scr->radius_ip)	/* no radius server declared, decline */
-    return DECLINED;
+  if (!rec->active || !scr->radius_ip)	/*  not active here, or no radius */
+    return DECLINED;                    /*  server declared, decline      */
   
   if ((res = get_basic_auth_pw(r, &sent_pw)))
     return res;
